<?php

/**
 * Return the main LM PayPal settings form.
 */
function lm_paypal_settings_form() {
  // TODO: add better support for switching to dev/sandbox site : maybe two sets
  // of options (defaulting to sane options), with a radio button to switch
  // between them.

  // pounard: Or, like the old simple_paypal module, 2 per default options,
  // sandbox and paypal in host, but do not let the choice to user to add
  // others?

  $site_mode = variable_get('lm_paypal_site_mode', 'test');

  $form['lm_paypal_site_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Site Mode'),
    '#options' => array(
      'live' => t('Live'), 
      'test' => t('Test'),
    ),
    '#required' => TRUE,
    '#default_value' => $site_mode,
    '#description' => t('Live mode will use the Live credentials and point users to www.paypal.com.  Test mode will use the Test (sandbox) credentials and point users to sandbox.paypal.com.'),
  );

  // Show these in order of most likely to be changed
  $site_name = variable_get('site_name', 'drupal');
  $form['live_account'] = array(
    '#type' => 'fieldset',
    '#title' => 'Live Account',
    '#collapsible' => TRUE,
    '#collapsed' => ($site_mode == 'test'),
  );
  $form['live_account']['lm_paypal_business'] = array(
    '#type' => 'textfield',
    '#title' => t('Live LM PayPal Business/Premier Email'),
    '#default_value' => variable_get('lm_paypal_business', ''),
    '#maxlength' => 100,
    '#description' => t('The <b>real</b> PayPal Business/Premier Email for the current website: %site_name', array('%site_name' => $site_name)),
  );
  
  $attrs = array('target' => '_blank');
  
  $form['test_account'] = array(
    '#type' => 'fieldset',
    '#title' => 'Test Account',
    '#collapsible' => TRUE,
    '#collapsed' => ($site_mode == 'live'),
  );
  
  $form['test_account']['lm_paypal_test_account'] = array(
    '#type' => 'textfield',
    '#title' => t('Test PayPal Credentials'),
    '#default_value' => variable_get('lm_paypal_test_account', ''),
    '#description' => t('A <b>sandbox</b> email account from PayPal.  Go to !link to get a PayPal test account.', array(
      '!link' => l('developer.paypal.com', 'http://developer.paypal.com', array('attributes' => $attrs)),
    )),
  );
  
  $form['lm_paypal_ipns_max_age'] = array(
    '#type' => 'textfield',
    '#title' => t('LM PayPal Max Age IPNS'),
    '#default_value' => variable_get('lm_paypal_ipns_max_age', LM_PAYPAL_IPNS_MAX_AGE_DEFAULT),
    '#maxlength' => 10,
    '#required' => FALSE, // False because NULL is zero here
    '#validate' => array('lm_paypal_is_integer_between' => array(1)),
    '#description' => t('Maximum age of an old IPN record in hours before it is deleted. Type 0 if you want to keep them forever.'),
  );

  $form['lm_paypal_cron_publish'] = array(
    '#type' => 'checkbox',
    '#title' => t('Cron Publish nodes'),
    '#default_value' => variable_get('lm_paypal_cron_publish', FALSE),
    '#description' => t('Normally you must set the workflow for your node types to "Published".  Using this option you can set them to "unPublished".  The cron process will publish them after payment is made and unpublish them when they expire.'),
  );

  $form['lm_paypal_expire_fudge'] = array(
    '#type' => 'checkbox',
    '#title' => t('Expire fudge'),
    '#default_value' => variable_get('lm_paypal_expire_fudge', FALSE),
    '#description' => t('Add a 24 hour fudge to the expire time.  Use if the cron runs only once per day or if you are using varnish or another cache server that only expires cache once per day.  This feature is only available when the "Cron Publish Node" is enabled.'),
  );

 $form['lm_paypal_prefix_nid'] = array(
    '#type' => 'checkbox',
    '#title' => t('Prefix node id'),
    '#default_value' => variable_get('lm_paypal_prefix_nid', FALSE),
    '#description' => t('Prefix paypal item description with node id. ie: 312/item description'),
  );

 $form['lm_paypal_prefix_hostname'] = array(
    '#type' => 'checkbox',
    '#title' => t('Prefix hostname'),
    '#default_value' => variable_get('lm_paypal_prefix_hostname', FALSE),
    '#description' => t('Prefix paypal item description with hostname. ie: hostname/item description'),
  );

 $form['lm_paypal_disable_myaccount'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable My Account display'),
    '#default_value' => variable_get('lm_paypal_disable_myaccount', FALSE),
    '#description' => t('Disable display in My Account of Paypal Payments and Subscription when no payments, no subscriptions and no nodes that need paid exist.'),
  );


  $form['lm_paypal_obey_test_ipns'] = array(
    '#type' => 'checkbox',
    '#title' => t('LM PayPal Obey Test IPNS'),
    '#default_value' => variable_get('lm_paypal_obey_test_ipns', LM_PAYPAL_OBEY_TEST_IPNS_DEFAULT),
    '#description' => t('Obey test IPNS, from PayPal Sandbox, as if real'),
  );

  $form['lm_paypal_skip_validation'] = array(
    '#type' => 'value',
    '#value' => variable_get('lm_paypal_skip_validation', FALSE),
    '#title' => t('Disable validation of payments with paypal.com'),
    '#description' => t('This should NEVER be enabled, except for testing. Hidden from the UI for safety'),
  );

  $form['lm_paypal_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('LM PayPal Debug'),
    '#default_value' => variable_get('lm_paypal_debug', LM_PAYPAL_DEBUG_DEFAULT),
    '#description' => t('Enabled verbose debugging output of LM PayPal'),
  );

  return system_settings_form($form);
}

function lm_paypal_settings_form_validate($form, &$form_state) {
  switch($form_state['values']['lm_paypal_site_mode']) {
    case 'live':
      $live_account = $form_state['values']['lm_paypal_business'];
      if (!valid_email_address($live_account)) {
        form_set_error('lm_paypal_business', t('A valid account must be specified when the site is in <em>Live</em> mode.'));
      }
      break;
    case 'test':
      $test_account = $form_state['values']['lm_paypal_test_account'];
      if (!valid_email_address($test_account)) {
        form_set_error('lm_paypal_test_account', t('A valid test account must be specified when the site is in <em>Test</em> mode.'));
      }
      break;
    default:
      form_set_error('lm_paypal_site_mode', t('Site mode must be <em>Live</em> or <em>Test</em>'));
  }
}

/**
 * View all saved IPNs
 *
 * Mostly borrowed from watchdog.module.
 */
function lm_paypal_ipns() {
  $ipns_per_page = 50;

  $output = drupal_get_form('lm_paypal_ipns_filter');

  $header = array(
    array('data' => t('Id'), 'field' => 'id'),
    array('data' => t('Date'), 'field' => 'timestamp', 'sort' => 'desc'),
    array('data' => t('Txn Type'), 'field' => 'txn_type'),
    array('data' => t('User'), 'field' => 'custom'), // TODO: not 'custom' anymore
  );

  $rows = array();

  $sql = "SELECT id, timestamp, txn_type, custom FROM {lm_paypal_ipns}";
  $tablesort = tablesort_sql($header);

  // If not sorting by timestamp then make that the 2nd field to sort on
  if (strpos($tablesort, 'timestamp') === FALSE) {
    $tablesort .= ', timestamp DESC';
  }

  $type = $_SESSION['lm_paypal_ipns_filter'];
  if ($type != 'all') {
    if (strpos($type, '%') === FALSE) {
      $result = pager_query($sql ." WHERE txn_type = '%s'". $tablesort, $ipns_per_page, 0, NULL, $type);
    }
    else {
      // If type contains a '%' use like to match it
      $result = pager_query($sql ." WHERE txn_type like '%s'". $tablesort, $ipns_per_page, 0, NULL, $type);
    }
  }
  else {
    $result = pager_query($sql . $tablesort, $ipns_per_page);
  }

  while ($ipn = db_fetch_object($result)) {
    $custom = lm_paypal_unpack_ipn_custom($ipn);
    // $uid = $custom['uid'];
    // $other = ($ipn->custom >> 16) & 0xFFFF;
    $custom_other = isset($custom['other']) ? $custom['other'] : '';
    $rows[] = array(
      'data' => array(
        l($ipn->id, "admin/settings/lm_paypal/id/$ipn->id"),
        format_date($ipn->timestamp, 'small'),
        check_plain($ipn->txn_type),
        $custom['uid'] . ($custom_other == '' ? '' : " ($custom_other)"),
      ),
    );
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No ipns found.'), 'colspan' => 3));
  }

  $output .= theme('table', $header, $rows);
  $output .= theme('pager', NULL, $ipns_per_page, 0);

  return $output;
}








/**
 * Display a saved IPN.
 *
 * @param $id
 *   Required. The id of the IPN to display.
 * @return
 *   The string containing the details of the IPN.
 */
function lm_paypal_id($id = '') {
  $id = check_plain($id);

  if ($id == '' || !is_numeric($id) || intval($id) != $id) {
    watchdog(LM_PAYPAL, 'Bad id passed: %id', array('%id' => $id), WATCHDOG_WARNING);
    return t('Huh?');
  }

  // Output the transaction as a table of fields/values (skip the empty ones)
  $output = '<h2>'. t('Transaction %id', array('%id' => $id)) .'</h2>';
  $header = array(t('field'), t('value'));
  $ipns = db_query("SELECT * FROM {lm_paypal_ipns} WHERE id = %d", $id);
  foreach (db_fetch_array($ipns) as $key => $value) {
    if ($value == '' || $key == 'ipn') {
      continue;
    }
    if ($key == 'timestamp') {
      $value = format_date($value);
    }
    else {
      $value = check_plain($value);
    }
    $rows[] = array('data' => array($key, $value));
  }
  $output .= theme('table', $header, $rows);
  return $output;
}



function lm_paypal_ipns_filter() {
  $names = array(
    'all' => t('all messages'),
    'web_accept' => t('donation/sent money (web_accept)'),
    'subscr_%' => t('all subscription IPNs'),
    'subscr_signup' => t('subscription signup'),
    'subscr_payment' => t('subscription payment'),
    'subscr_cancel' => t('subscription cancel'),
    'subscr_eot' => t('subscription eot'),
  );

// TODO: seems like we should be using form state for this?
  if (empty($_SESSION['lm_paypal_ipns_filter'])) {
    $_SESSION['lm_paypal_ipns_filter'] = 'all';
  }

  $form['#submit'] = array('lm_paypal_ipns_submit');

  $form['filter'] = array(
    '#type' => 'select',
    '#title' => t('Filter IPN type'),
    '#options' => $names,
    '#default_value' => $_SESSION['lm_paypal_ipns_filter'],
  );
  $form['#action'] = url('admin/settings/lm_paypal/ipns');

  $form['submit'] = array('#type' => 'submit', '#value' => t('Filter'));

  return $form;
}


/**
 * Process the form submission for lm_paypal_ipns
 */
function lm_paypal_ipns_submit($form, &$form_state) {
  $_SESSION['lm_paypal_ipns_filter'] = $form_state['values']['filter'];
}
